# Profile for configuring apache to proxy requests for the console service.
#
#
# @param certname [String] The certname the console will use to encrypt network traffic.
# @param trapperkeeper_proxy_listen_address [String] The network interface used by the
#        trapperkeeper proxy.
# @param trapperkeeper_proxy_listen_port [Integer] The port that the trapperkeeper proxy
#        is listening on.
# @param ssl_listen_address [String] The network interface used for ssl connections.
# @param ssl_listen_port [Integer] The port used for ssl connections.
# @param browser_ssl_cert [String] Sets the path to the server certificate PEM file used
#        by the web service.
# @param browser_ssl_private_key [String] For use with a custom CA, the path to a private
#        key for your public ca certificate.
# @param browser_ssl_cert_chain [String] For use with a custom CA, the path to the ca certificate.
# @param browser_ssl_ca_cert [String] For use with a custom CA, the path to the certificate.
class puppet_enterprise::profile::console::apache_proxy (
  $certname = $::clientcert,
  $trapperkeeper_proxy_listen_address = $puppet_enterprise::params::plaintext_address,
  $trapperkeeper_proxy_listen_port    = $puppet_enterprise::params::console_services_listen_port,
  $ssl_cipher_suite        = 'ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:kEDH+AESGCM:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES256-SHA:DHE-RSA-AES128-SHA256:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA:DHE-RSA-AES256-SHA:AES128-GCM-SHA256:AES256-GCM-SHA384:AES128-SHA256:AES256-SHA256:AES128-SHA:AES256-SHA:AES:CAMELLIA:DES-CBC3-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA',
  $ssl_listen_address      = $puppet_enterprise::params::ssl_address,
  $ssl_listen_port         = $puppet_enterprise::console_port,
  $browser_ssl_cert        = undef,
  $browser_ssl_private_key = undef,
  $browser_ssl_cert_chain  = undef,
  $browser_ssl_ca_cert     = undef,
  $dhparam_file            = '/opt/puppet/share/puppet-dashboard/certs/dhparam_puppetproxy.pem' ,
  $dhparam_cert_group      = 'puppet-dashboard',
  $dhparam_cert_mode       = '0644',
  $dhparam_cert_owner      = 'puppet-dashboard',
) inherits puppet_enterprise {
  $cert_dir = '/opt/puppet/share/console-services/certs'

  # puppet_enterprise::certs { 'pe-console-services' : ... } -> creates these files
  if $browser_ssl_cert != undef and $browser_ssl_private_key != undef {
    $ssl_key  = $browser_ssl_private_key
    $ssl_cert = $browser_ssl_cert
  } else {
    $ssl_key  = "${cert_dir}/${certname}.private_key.pem"
    $ssl_cert = "${cert_dir}/${certname}.cert.pem"
  }

  $dhparam_base_dir = '/opt/puppet/share/puppet-dashboard/certs'

  $default_dhparam_file = "${dhparam_base_dir}/dhparam_puppetproxy.pem"
  file { $default_dhparam_file:
    ensure => file,
    source => 'puppet:///modules/puppet_enterprise/console/dhparam_puppetproxy.pem',
    owner  => $dhparam_cert_owner,
    group  => $dhparam_cert_group,
    mode   => $dhparam_cert_mode,
  }

  $dhparam_cert_target = "${dhparam_base_dir}/dhparam_puppetproxy_cert.pem"

  puppet_enterprise::console::dhparam { $dhparam_file:
    dhparam_cert_target     => $dhparam_cert_target,
    ssl_certificate_file    => $ssl_cert ,
    dhparam_cert_group      => $dhparam_cert_group,
    dhparam_cert_mode       => $dhparam_cert_mode,
    dhparam_cert_owner      => $dhparam_cert_owner,
    require                 => File['/opt/puppet/share/puppet-dashboard/certs', $default_dhparam_file],
  }

  if $browser_ssl_cert_chain != undef {
    $ssl_cert_chain = $browser_ssl_cert_chain
  } else {
    $ssl_cert_chain = $puppet_enterprise::params::localcacert
  }

  if $browser_ssl_ca_cert != undef {
    $ssl_ca_cert = $browser_ssl_ca_cert
  } else {
    $ssl_ca_cert = $puppet_enterprise::params::localcacert
  }

  # Uses
  # $dhparams_file
  # $trapperkeeper_proxy_listen_address
  # $trapperkeeper_proxy_listen_port
  # $ssl_key
  # $ssl_cert
  # $ssl_cert_chain
  # $ssl_ca_cert
  # $ssl_listen_address
  # $ssl_listen_port
  file { '/etc/puppetlabs/httpd/conf.d/puppetproxy.conf':
    owner   => $puppet_enterprise::params::root_user,
    group   => $puppet_enterprise::params::root_group,
    mode    => '0644',
    content => template('puppet_enterprise/profile/console-services/puppetproxy.conf.erb'),
    notify  => Service['pe-httpd'],
    require => [Puppet_enterprise::Certs['pe-console-services::server_cert'], Puppet_enterprise::Console::Dhparam[$dhparam_file]],
  }
}
