# Profile for configuring console services.
#
#
# @param certname [String] The certname the console will use to encrypt network traffic.
# @param listen_address [String] The network interface used by the console service.
# @param ssl_listen_address [String] The network interface used by console services for ssl connections.
# @param listen_port [Integer] The port that console services is listening on.
# @param ssl_listen_port [Integer] The port that console services is listening on for ssl
#        connections.
# @param api_listen_port [Integer] The port that the console services api is listening on.
# @param api_ssl_listen_port [Integer] The port that console services is listening on for ssl
#        connections.
# @param activity_url_prefix [String] The url prefix for the activity api.
# @param classifier_url_prefix [String] The url prefix for the classifier api.
# @param rbac_url_prefix [String] The url prefix for the rbac api.
# @param localcacert [String] The path to the local CA certificate. This will be used instead of the
#        CA that is in Puppet's ssl dir.
# @param tk_jetty_max_threads_api [Integer] Maximum number of threads that jetty will devote to the
#        console api. Default 100.
# @param tk_jetty_max_threads_console [Integer] Maximum number of threads that jetty will devote to
#        the console ui. Default 100.
# @param tk_jetty_request-header_max_size [Integer] Sets the maximum size of an HTTP Request Header.
class puppet_enterprise::profile::console::console_services_config (
  $certname                         = $::clientcert,
  $listen_address                   = $puppet_enterprise::params::plaintext_address,
  $ssl_listen_address               = $puppet_enterprise::params::ssl_address,
  $listen_port                      = $puppet_enterprise::params::console_services_listen_port,
  $ssl_listen_port                  = $puppet_enterprise::params::console_services_ssl_listen_port,
  $api_listen_port                  = $puppet_enterprise::params::console_services_api_listen_port,
  $api_ssl_listen_port              = $puppet_enterprise::api_port,
  $activity_url_prefix              = $puppet_enterprise::activity_url_prefix,
  $classifier_url_prefix            = $puppet_enterprise::classifier_url_prefix,
  $rbac_url_prefix                  = $puppet_enterprise::rbac_url_prefix,
  $localcacert                      = $puppet_enterprise::params::localcacert,
  $tk_jetty_max_threads_api         = undef,
  $tk_jetty_max_threads_console     = undef,
  $tk_jetty_request_header_max_size = 65536,
) inherits puppet_enterprise {

  File {
    ensure => present,
    owner  => 'pe-console-services',
    group  => 'pe-console-services',
    mode   => '0640',
  }

  Pe_concat {
    owner  => 'pe-console-services',
    group  => 'pe-console-services',
    mode   => '0640',
  }

  $cert_dir = '/opt/puppet/share/console-services/certs'
  # puppet_enterprise::certs { 'pe-console-services' : ... } -> creates these files
  $ssl_key = "${cert_dir}/${certname}.private_key.pem"
  $ssl_cert =  "${cert_dir}/${certname}.cert.pem"

  $confdir = '/etc/puppetlabs/console-services'
  $access_log_config = "${confdir}/request-logging.xml"
  # Uses
  #   $listen_address
  #   $listen_port
  #   $ssl_listen_address
  #   $ssl_listen_port
  #   $ssl_key
  #   $ssl_cert
  #   $localcacert
  #   $access_log_config
  #   $api_listen_port
  #   $api_ssl_listen_port
  #   $classifier_url_prefix
  #   $rbac_url_prefix
  #   $activity_url_prefix
  #   $tk_jetty_max_threads_console
  #   $tk_jetty_max_threads_api
  #   $tk_jetty_request_header_max_size
  file { "${confdir}/conf.d/webserver.conf":
    content => template('puppet_enterprise/profile/console-services/webserver.conf.erb'),
  }

  $log_config = "${confdir}/logback.xml"
  # Uses
  #   $log_config
  file { "${confdir}/conf.d/global.conf":
    content => template('puppet_enterprise/profile/console-services/global.conf.erb'),
  }

  pe_concat { "${confdir}/bootstrap.cfg" : }
}
